﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinDataSource;
using Infragistics.Win.UltraWinGrid;
using System.Globalization;
using System.Collections;
using Infragistics.Win;

namespace ServicePlan
{
    public partial class FormUltraDataSource : Form
    {
        // RowTypes (enum Wert Reihenfolge für Sortierung)
        private enum RowTypes
        {
            DayNoteRow      = 0,
            PlanningRow     = 1,
            ServiceNoteRow  = 2,
        }

        private enum ViewTypes
        {
            Standard        = 0,
            Detail          = 1,
        }

        private DateTime fromDate = new DateTime( 2011, 1, 1 );
        private DateTime toDateFull = new DateTime( 2011, 12, 31 );
        private DateTime toDateDetail = new DateTime( 2011, 1, 2 );
        private int departmentCount = 3;
        private int emplCount = 20;
        private int planningCount = 1;
        private int planningMaxCount = 3;
        private int serviceNoteCount = 1;
        private int dayNoteCount = 0;
        private int dayNoteMaxCount = 10;
        private RowTag newRowTag = null;
        private ViewTypes viewType = ViewTypes.Standard;
        private FormZB fZB = null;


        #region Private Classes
        private class RowTag
        {
            public RowTag( RowTypes type )
            {
                this.RowType = type;
                this.Id = 0;
                this.TypeIndex = 0;
            }
            public RowTag( RowTypes type, int id )
            {
                this.RowType = type;
                this.Id = id;
                this.TypeIndex = 0;
            }
            public RowTag( RowTypes type, int id, int typeIndex )
            {
                this.RowType = type;
                this.Id = id;
                this.TypeIndex = typeIndex;
            }
            public RowTypes RowType;
            public int Id = 0;
            public int TypeIndex =0;
        }

        private class GridSortComparer : IComparer
        {
            internal GridSortComparer()
            {
            }

            int IComparer.Compare( object x, object y )
            {

                // Passed in objects are cells. So you have to typecast them to UltraGridCell objects first.
                UltraGridCell xCell = (UltraGridCell)x;
                UltraGridCell yCell = (UltraGridCell)y;

                // Do your own comparision between the values of xCell and yCell and return a negative 
                // number if xCell is less than yCell, positive number if xCell is greater than yCell, 
                // and 0 if xCell and yCell are equal.
                RowTag xTag = (RowTag)xCell.Row.Tag;
                RowTag yTag = (RowTag)yCell.Row.Tag;

                // Unterschiedlicher Typ
                if ( xTag.RowType != yTag.RowType )
                    return (int)xTag.RowType < (int)yTag.RowType ? -1 : 1;

                // Gleicher Typ Band 0 -> Tagesnotiz oder Abteilung
                if ( xCell.Band.Key.Equals( "Band 0" ) )
                {
                    // Abteilungsname
                    if ( xTag.RowType == RowTypes.PlanningRow )
                        return String.Compare( xCell.Text, yCell.Text, true );
                    // Tagesnotiz-Index
                    if ( xTag.TypeIndex != yTag.TypeIndex )
                        return xTag.TypeIndex < yTag.TypeIndex ? -1 : 1;

                    return 0;
                }

                // Gleicher Typ Band 1: Mitarbeiter
                if ( xCell.Band.Key.Equals( "Band 1" ) )
                {
                    // Gleicher MA -> TypeIndex vergleichen
                    if ( xTag.Id == yTag.Id )
                    {
                        if ( xTag.TypeIndex != yTag.TypeIndex )
                            return xTag.TypeIndex < yTag.TypeIndex ? -1 : 1;
                        return 0;
                    }
                    // unterschiedliche MA -> Name vergleichen
                    return String.Compare( xCell.Text, yCell.Text, true );
                }

                // Gleicher Typ Band 1: Dienstnotizen
                if ( xCell.Band.Key.Equals( "Band 2" ) )
                {
                    // Dienstnotiz-Index
                    if ( xTag.TypeIndex != yTag.TypeIndex )
                        return xTag.TypeIndex < yTag.TypeIndex ? -1 : 1;

                    return 0;
                }

                // Sind gleich
                return 0;
            }
        }

        #endregion

        #region Konstruktor & Initialisierung

        public FormUltraDataSource()
        {
            InitializeComponent();
        }

        private void FormByUltraDataSource_Load( object sender, EventArgs e )
        {
            // Struktur und Daten
            this.InitUltraDataSource( this.DataSource, this.toDateFull, false );
            this.InitUltraDataSource( this.DataSourceDetail, this.toDateDetail, true );
            // An das Grid Binden
            this.Grid.DataSource = this.DataSource;
            this.GridDetail.DataSource = this.DataSourceDetail;
            // Daten einfügen
            this.InitData( this.DataSource, this.toDateFull, false);            // Standard
            this.InitData( this.DataSourceDetail, this.toDateDetail, true );    // Detail
        }

        private void InitUltraDataSource( UltraDataSource ds, DateTime endDate, bool isDetail )
        {
            string key = "";

            // Band 0 (Abteilung)
            // ---------------------------------------------------------------------------------------
            UltraDataBand rootBand = ds.Band;
            rootBand.Key = "Band 0";
            rootBand.Columns.Add( "Organisation", typeof( string ) );
            for ( DateTime date = this.fromDate; date <= endDate; date = date.AddDays( 1.0 ) )
            {
                key = date.ToShortDateString();
                ds.Band.Columns.Add( key, typeof( string ) );
            }
            // Detailview: 24 Std. im 15 Minutenraster
            if ( isDetail )
            {
                DateTimeFormatInfo dateTimeFormatInfo = System.Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat;
                DateTime hour = new DateTime( this.fromDate.Year, this.fromDate.Month, this.fromDate.Day, 0, 0, 0 );
                for ( int i=0; i < 4 * 24; i++ )
                {
                    key = hour.ToString( dateTimeFormatInfo.ShortTimePattern );
                    ds.Band.Columns.Add( key, typeof( string ) );
                    hour = hour.AddMinutes( 15.0 );
                }
            }

            // Band 1 (Mitarbeiter)
            // ---------------------------------------------------------------------------------------
            UltraDataBand emplBand = ds.Band.ChildBands.Add( "Band 1" );
            emplBand.Columns.Add( "Organisation", typeof( string ) );
            for ( DateTime date = this.fromDate; date <= endDate; date = date.AddDays( 1.0 ) )
            {
                key = date.ToShortDateString();
                UltraDataColumn c = emplBand.Columns.Add( key, typeof( string ) );
            }
            // Detailview: 24 Std. im 15 Minutenraster
            if ( isDetail )
            {
                DateTimeFormatInfo dateTimeFormatInfo = System.Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat;
                DateTime hour = new DateTime( this.fromDate.Year, this.fromDate.Month, this.fromDate.Day, 0, 0, 0 );
                for ( int i=0; i < 4 * 24; i++ )
                {
                    key = hour.ToString( dateTimeFormatInfo.ShortTimePattern );
                    emplBand.Columns.Add( key, typeof( string ) );
                    hour = hour.AddMinutes( 15.0 );
                }
            }

            // Band 2 (Dienstnotizen)
            // ---------------------------------------------------------------------------------------
            UltraDataBand noteBand = emplBand.ChildBands.Add( "Band 2" );
            noteBand.Columns.Add( "Organisation", typeof( string ) );
            for ( DateTime date = this.fromDate; date <= endDate; date = date.AddDays( 1.0 ) )
            {
                key = date.ToShortDateString();
                noteBand.Columns.Add( key, typeof( string ) );
            }
            // Detailview: 24 Std. im 15 Minutenraster
            if ( isDetail )
            {
                DateTimeFormatInfo dateTimeFormatInfo = System.Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat;
                DateTime hour = new DateTime( this.fromDate.Year, this.fromDate.Month, this.fromDate.Day, 0, 0, 0 );
                for ( int i=0; i < 4 * 24; i++ )
                {
                    key = hour.ToString( dateTimeFormatInfo.ShortTimePattern );
                    noteBand.Columns.Add( key, typeof( string ) );
                    hour = hour.AddMinutes( 15.0 );
                }
            }
        }

        private void InitData( UltraDataSource ds, DateTime endDate, bool isDetail )
        {
            string key = "";
            // Band 0 (Abteilung)
            // ----------------------------------------------------------------------------------------
            for ( int depIndex = 0; depIndex < departmentCount; depIndex++ )
            {
                // Add row -> AddRow-event der DataSource setzt den RowTag. Der Initialize_row event des Grid tritt "später"
                // auf (wenn repainted).
                this.newRowTag = new RowTag( RowTypes.PlanningRow );
                UltraDataRow depRow = ds.Rows.Add( true );
                depRow["Organisation"] = "Department " + depIndex.ToString();

                // Band 1 (Planung)
                // ------------------------------------------------------------------------------------
                UltraDataRowsCollection emplRows = depRow.GetChildRows( "Band 1" );
                for ( int emplIndex = 0; emplIndex < emplCount; emplIndex++ )
                {
                    this.newRowTag = new RowTag( RowTypes.PlanningRow, emplIndex + 1, 0 );
                    UltraDataRow emplRow = emplRows.Add( true );
                    emplRow["Organisation"] = string.Format( "  Empl_{0}", emplIndex.ToString() );
                    for ( DateTime date = this.fromDate; date <= endDate; date = date.AddDays( 1.0 ) )
                    {
                        key = date.ToShortDateString();
                        emplRow[key] = string.Format( "S{0}.{1}", date.Month, date.Day );
                    }

                    // Band 2 (Dienstnotizen)
                    // --------------------------------------------------------------------------------
                    UltraDataRowsCollection noteRows = emplRow.GetChildRows( "Band 2" );
                    for ( int noteIndex = 0; noteIndex < serviceNoteCount; noteIndex++ )
                    {
                        this.newRowTag = new RowTag( RowTypes.ServiceNoteRow, 0, noteIndex );
                        UltraDataRow serviceNoteRow = noteRows.Add( true );
                        ( (RowTag)serviceNoteRow.Tag ).Id = ( (RowTag)serviceNoteRow.ParentRow.Tag ).Id;    // parentRow -> Empl Id
                        for ( DateTime date = this.fromDate; date <= endDate; date = date.AddDays( 1.0 ) )
                        {
                            key = date.ToShortDateString();
                            if ( date == fromDate.AddDays( 4 ) )
                                serviceNoteRow[key] = "Multiline cell text abc";
                            else
                                serviceNoteRow[key] = string.Format( "N{0}.{1}", date.Month, date.Day );
                        }
                    }

                }

            }
        }

        #endregion

        #region Grid Event-Handlers

        private void Grid_InitializeLayout( object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e )
        {
//            this.GridInitLayout( this.Grid, e, this.toDateFull, false );

            // Variante DataSource wechseln
            if ( this.viewType == ViewTypes.Detail )
                this.GridInitLayout( this.Grid, e, this.toDateDetail, true );
            else
                this.GridInitLayout( this.Grid, e, this.toDateFull, false );
        }

        private void Grid_InitializeRow( object sender, InitializeRowEventArgs e )
        {
//            this.GridInitRow( this.Grid, e, false );
            this.GridInitRow( this.Grid, e, this.viewType == ViewTypes.Detail );
        }

        private void GridDetail_InitializeLayout( object sender, InitializeLayoutEventArgs e )
        {
            this.GridInitLayout( this.GridDetail, e, this.toDateDetail, true );
        }

        private void GridDetail_InitializeRow( object sender, InitializeRowEventArgs e )
        {
            this.GridInitRow( this.GridDetail, e, true );
        }

        private void GridInitLayout( Infragistics.Win.UltraWinGrid.UltraGrid grid, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e, 
            DateTime toDate, bool isDetail )
        {
            // Standard settings helper class
            e.Layout.MaxBandDepth = 3;
            e.Layout.ViewStyle = Infragistics.Win.UltraWinGrid.ViewStyle.MultiBand;
            e.Layout.AutoFitStyle = AutoFitStyle.None;
            e.Layout.Override.SelectTypeCell = SelectType.Extended;
            e.Layout.Override.SelectTypeRow = SelectType.Extended;
            e.Layout.Override.CellClickAction = CellClickAction.EditAndSelectText;
            e.Layout.Override.HeaderClickAction = HeaderClickAction.Select;
            e.Layout.Override.SelectTypeCol = SelectType.Extended;
            e.Layout.Override.RowSelectors = Infragistics.Win.DefaultableBoolean.False;
            e.Layout.Override.AllowAddNew = AllowAddNew.No;
            e.Layout.Override.AllowUpdate = Infragistics.Win.DefaultableBoolean.True;
            e.Layout.Override.AllowDelete = Infragistics.Win.DefaultableBoolean.False;
            e.Layout.Override.AllowColMoving = AllowColMoving.NotAllowed;
            e.Layout.Override.AllowColSwapping = AllowColSwapping.NotAllowed;
            e.Layout.Override.AllowColSizing = AllowColSizing.None;
            e.Layout.Override.ColumnAutoSizeMode = ColumnAutoSizeMode.None;

            // Scrolling
            e.Layout.ScrollStyle = ScrollStyle.Immediate;
            e.Layout.ScrollBounds = ScrollBounds.ScrollToLastItem;

            // Keine punktierten Connectors vom Plus-Sign zum Band etc.
            e.Layout.RowConnectorStyle = RowConnectorStyle.None;
            // Kein Abstand zwischen den Bands
            e.Layout.InterBandSpacing = 0;

            // Fixed Rows Properties (Day notes)
            e.Layout.Override.FixedRowStyle = FixedRowStyle.Top;
            e.Layout.Override.FixedRowIndicator = FixedRowIndicator.None;
            e.Layout.Override.FixedRowAppearance.BackColor = Color.Azure;
            e.Layout.Override.SpecialRowSeparator = SpecialRowSeparator.FixedRows;
            e.Layout.Override.SpecialRowSeparatorAppearance.BackColor = Color.Blue;
            e.Layout.Override.SpecialRowSeparatorHeight = 4;
            e.Layout.Override.BorderStyleSpecialRowSeparator = UIElementBorderStyle.RaisedSoft;
            e.Layout.Override.FixedRowsLimit = dayNoteMaxCount;
            e.Layout.Override.DefaultColWidth = 50;

            // Disable some AppStyling for highlite rows
            e.Layout.Override.ActiveAppearancesEnabled = DefaultableBoolean.False;
            e.Layout.Override.SelectedRowAppearance.Reset();

            // Fixed Columns
            e.Layout.UseFixedHeaders = true;
            e.Layout.Override.FixedHeaderIndicator = FixedHeaderIndicator.None;

            // Band 0 (Departments) -----------------------------------------------------------
            UltraGridBand band0 = e.Layout.Bands["Band 0"];
            band0.Override.HeaderPlacement = HeaderPlacement.OncePerRowIsland;
            band0.ColHeaderLines = 3;

            // Organisations-column
            band0.Columns["Organisation"].Header.Caption = "\n\nOrganisation";
            band0.Columns["Organisation"].Header.Fixed = true;
            band0.Columns["Organisation"].Header.Appearance.BackColor = Color.WhiteSmoke;
            band0.Columns["Organisation"].Width = 150;

            // Band 1 ( employees / planning ) ------------------------------------------------
            UltraGridBand band1 = e.Layout.Bands["Band 1"];
            band1.Indentation = 0;   // keine Einrückung
            band1.ColHeadersVisible = false;
            band1.Columns["Organisation"].Width = 150;
            band1.Override.CellAppearance.BackColor = band1.Header.Appearance.BackColor;
            band1.Columns["Organisation"].Header.Fixed = true;
            band1.Columns["Organisation"].MergedCellStyle = MergedCellStyle.OnlyWhenSorted;

            // Sortierung
            band1.Columns["Organisation"].SortComparer = new GridSortComparer();
            band1.SortedColumns.Add( "Organisation", false, false );

            // Band 2 ( task notes ) --------------------------------------------------------
            UltraGridBand band2 = e.Layout.Bands["Band 2"];
            band2.Indentation = 0;   // keine Einrückung
            band2.ColHeadersVisible = false;
            band2.Columns["Organisation"].Width = 150;

            // align cell size to biggest task note
            band2.Override.RowSizingArea = RowSizingArea.EntireRow;
            band2.Override.RowSizing = RowSizing.AutoFree;
            band2.Override.CellMultiLine = DefaultableBoolean.True;

            // detail view settings (fix day columns)
            if ( isDetail )
            {
                string key = "";
                for ( DateTime date = fromDate; date <= toDate; date = date.AddDays( 1.0 ) )
                {
                    key = date.ToShortDateString();
                    band0.Columns[key].Header.Fixed = true;
                    band1.Columns[key].Header.Fixed = true;
                    band2.Columns[key].Header.Fixed = true;
                }
                DateTimeFormatInfo dateTimeFormatInfo = System.Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat;
                DateTime hour = new DateTime( this.fromDate.Year, this.fromDate.Month, this.fromDate.Day, 0, 0, 0 );
                for ( int i=0; i < 4 * 24; i++ )
                {
                    key = hour.ToString( dateTimeFormatInfo.ShortTimePattern );
                    band0.Columns[key].Header.Caption = "\nh:mm\nSa";
                    band0.Columns[key].Width = 15;
                    band1.Columns[key].Width = 15;
                    band2.Columns[key].Width = 15;
                    hour = hour.AddMinutes( 15.0 );
                }
            }

            // Jeden Tag mit Monat, Tag, Wochentag
            CultureInfo cultureInfo = System.Threading.Thread.CurrentThread.CurrentUICulture;
            for ( DateTime date = fromDate; date <= toDate; date = date.AddDays( 1.0 ) )
            {
                // Kalender
                string caption = string.Format( "{0}\n{1}.\n{2}", cultureInfo.DateTimeFormat.AbbreviatedMonthNames[date.Month - 1],
                    date.Day,
                    cultureInfo.DateTimeFormat.AbbreviatedDayNames[(int)date.DayOfWeek] );
                band0.Columns[date.ToShortDateString()].Header.Caption = caption;
                band0.Columns[date.ToShortDateString()].Header.Appearance.BackColor = date.Month % 2 == 0 ? Color.WhiteSmoke : Color.Gainsboro;

                // Aktuelles Datum
                if ( date == DateTime.Today )
                {
                    band0.Columns[date.ToShortDateString()].Header.Appearance.BackColor = Color.Black;
                    band0.Columns[date.ToShortDateString()].Header.Appearance.ForeColor = Color.White;
                    band0.Columns[date.ToShortDateString()].Header.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
                }

                if ( date.DayOfWeek == DayOfWeek.Saturday || date.DayOfWeek == DayOfWeek.Sunday )
                {
                    string key = date.ToShortDateString();

                    band0.Columns[key].Header.Appearance.BackColor = Color.LightYellow;

                    band0.Columns[key].CellAppearance.BackColor = Color.LightYellow;
                    band1.Columns[key].CellAppearance.BackColor = Color.LightYellow;
                    band2.Columns[key].CellAppearance.BackColor = Color.LightYellow;
                }
                if ( date == fromDate.AddDays( 4 ) )
                {
                    string key = date.ToShortDateString();

                    band2.Columns[key].CellMultiLine = DefaultableBoolean.True;
                }
            }
        }

        private void GridInitRow( Infragistics.Win.UltraWinGrid.UltraGrid grid, InitializeRowEventArgs e, bool isDetail )
        {
            UltraDataRow dataRow = (UltraDataRow)e.Row.ListObject;
            e.Row.Tag = dataRow.Tag;
            RowTypes rowType = ( (RowTag)e.Row.Tag ).RowType;

            ValueList propList = null;
            ValueList noteList = null;
            if ( this.Grid.DisplayLayout.ValueLists.Exists( "PropList" ) )
                propList = this.Grid.DisplayLayout.ValueLists["PropList"];
            else
            {
                propList = this.Grid.DisplayLayout.ValueLists.Add( "PropList" );
                propList.ValueListItems.Add( 1, "Prop 1" );
                propList.ValueListItems.Add( 2, "Prop 2" );
                propList.ValueListItems.Add( 3, "Prop 3" );
                propList.ValueListItems.Add( 4, "Prop 4" );
                propList.ValueListItems.Add( 5, "Prop 5" );
            }
            if ( this.Grid.DisplayLayout.ValueLists.Exists( "NoteList" ) )
                noteList = this.Grid.DisplayLayout.ValueLists["NoteList"];
            else
            {
                noteList = this.Grid.DisplayLayout.ValueLists.Add( "NoteList" );
                noteList.ValueListItems.Add( 1, "Note 1" );
                noteList.ValueListItems.Add( 2, "Note 2" );
                noteList.ValueListItems.Add( 3, "Note 3" );
                noteList.ValueListItems.Add( 4, "Note 4" );
                noteList.ValueListItems.Add( 5, "Note 5" );
            }

            if ( e.Row.Band.Key.Equals( "Band 0" ) )
            {
                switch ( rowType )
                {
                    case RowTypes.PlanningRow:   // Abteilung
                        // Name fett
                        e.Row.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
                        // Spalte MA-Property
                        if ( e.Row.Cells.Exists( "Property" ) )
                        {
                            // Fixieren und Breite
                            e.Row.Cells["Organisation"].Column.Header.Fixed = true;
                            e.Row.Cells["Property"].Column.Header.Fixed = true;
                            e.Row.Cells["Property"].Column.Header.Caption = "\n\nEigenschaft";
                            e.Row.Cells["Property"].Column.Width = e.Row.Cells[2].Width;
                            // DropDown setzen
                            e.Row.Cells["Property"].Column.ValueList = propList;
                            e.Row.Cells["Property"].Column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList;
                        }
                        break;

                    case RowTypes.DayNoteRow:   // Tagesnotiz
                        // Oben Fixieren
                        e.Row.Fixed = true;
                        // DropDown setzen
                        e.Row.Cells["Organisation"].Column.ValueList = noteList;
                        e.Row.Cells["Organisation"].Column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList;
                        break;
                }
            }

            // Planung
            if ( e.Row.Band.Key.Equals( "Band 1" ) )
            {
                // Spalte MA-Property
                if ( e.Row.Cells.Exists( "Property" ) )
                {
                    // Fixieren
                    e.Row.Cells["Organisation"].Column.Header.Fixed = true;
                    e.Row.Cells["Property"].Column.Header.Fixed = true;
                    e.Row.Cells["Property"].Column.Width = e.Row.Cells[2].Width;
                    // Propertytext setzen
                    e.Row.Cells["Property"].SetValue( "100%", false );
                }
                // Detailview: 24 Std. im 15 Minutenraster
                if ( isDetail )
                {
                    DateTimeFormatInfo dateTimeFormatInfo = System.Threading.Thread.CurrentThread.CurrentUICulture.DateTimeFormat;
                    DateTime hour = new DateTime( this.fromDate.Year, this.fromDate.Month, this.fromDate.Day, 0, 0, 0 );
                    int emplIndex = ( (RowTag)e.Row.Tag ).Id;
                    for ( int i=0; i < 4 * 24; i++ )
                    {
                        string key = hour.ToString( dateTimeFormatInfo.ShortTimePattern );
                        if ( emplIndex % 2 == 0 )
                        {
                            if ( i > 27 && i < 56 )
                                e.Row.Cells[key].Appearance.BackColor = Color.SaddleBrown;
                        }
                        else if ( emplIndex % 3 == 0 )
                        {
                            if ( i > 40 && i < 80 )
                                e.Row.Cells[key].Appearance.BackColor = Color.SaddleBrown;
                        }
                        else if ( emplIndex % 5 == 0 )
                        {
                            if ( ( i > 20 && i < 40 ) || ( i > 65 && i < 75 ) )
                                e.Row.Cells[key].Appearance.BackColor = Color.SaddleBrown;
                        }
                        else
                            if ( i > 40 && i < 80 )
                                e.Row.Cells[key].Appearance.BackColor = Color.SaddleBrown;

                        hour = hour.AddMinutes( 15.0 );
                    }
                }
            }

            // Dienstnotizen: Hintergrundfarbe
            if ( e.Row.Band.Key.Equals( "Band 2" ) )
            {
                // Hintergrundfareb setzen
                e.Row.Appearance.BackColor = Color.WhiteSmoke;
                // Zeilenhöhe an aktuellen Inhalt anpassen
                e.Row.PerformAutoSize();

                // Spalte MA-Property
                if ( e.Row.Cells.Exists( "Property" ) )
                {
                    // Fixieren
                    e.Row.Cells["Organisation"].Column.Header.Fixed = true;
                    e.Row.Cells["Property"].Column.Header.Fixed = true;
                    e.Row.Cells["Property"].Column.Width = e.Row.Cells[2].Width;
                }
            }
        }

        #endregion

        #region UltraDataSource Event-Handlers

        private void DataSource_RowAdded( object sender, RowAddedEventArgs e )
        {
            e.Row.Tag = this.newRowTag;
        }

        private void DataSourceDetail_RowAdded( object sender, RowAddedEventArgs e )
        {
            e.Row.Tag = this.newRowTag;
        }

        #endregion

        #region Service Notes
        private void BtnServiceNoteOn_Click( object sender, EventArgs e )
        {
            // Dienstnotizen anzeigen

            // Fullview
            IEnumerable enumerator = this.Grid.Rows.GetRowEnumerator( GridRowType.DataRow, this.Grid.DisplayLayout.Bands["Band 1"], 
                this.Grid.DisplayLayout.Bands["Band 1"] );
            foreach ( UltraGridRow row in enumerator )
                row.Expanded = true;
        }

        private void BtnServiceNoteOff_Click( object sender, EventArgs e )
        {
            // Dienstnotizen ausblenden
            IEnumerable enumerator = this.Grid.Rows.GetRowEnumerator( GridRowType.DataRow, this.Grid.DisplayLayout.Bands["Band 1"], 
                this.Grid.DisplayLayout.Bands["Band 1"] );
            foreach ( UltraGridRow row in enumerator )
                row.Expanded = false;
        }
        #endregion

        #region Tagesnotizen

        private void BtnDayNotePlus_Click( object sender, EventArgs e )
        {
            // Tagesnotizen erscheinen vor der ersten Abteilung
            // ------------------------------------------------
            if ( dayNoteCount < dayNoteMaxCount )
            {
                dayNoteCount++;
                // Neue Zeile -> initializeRow-event fixiert die Zeile oben
                this.newRowTag = new RowTag( RowTypes.DayNoteRow );
                // Fullview
                UltraDataRow dayNoteRow = this.DataSource.Rows.Add( true );
                dayNoteRow["Organisation"] = "Tagesnotiz " + dayNoteRow.Index.ToString();
            }
            // Fullview: Spaltentitel aktualisieren
            this.Grid.DisplayLayout.Bands[0].Columns["Organisation"].Header.Caption = 
                dayNoteCount > 0 ? "\nTagesnotizen\nOrganisation" : "\n\nOrganisation";
        }

        private void BtnDayNoteMinus_Click( object sender, EventArgs e )
        {
            if ( dayNoteCount > 0 )
            {
                dayNoteCount--;
                // Die Position in der UltraDataSource ist nicht 0.
                // Die Zeile muss im TopMostRows-Collection gesucht werden !?
                int lastNoteIndex = -1;
                for ( int index=0; index < this.DataSource.Rows.Count; index++ )
                {
                    if ( ((RowTag)this.DataSource.Rows[index].Tag).RowType == RowTypes.DayNoteRow )
                        lastNoteIndex = index;
                }
                if ( lastNoteIndex >= 0 )
                    this.DataSource.Rows.RemoveAt( lastNoteIndex );
            }
            // Spaltentitel aktualisieren
            this.Grid.DisplayLayout.Bands[0].Columns["Organisation"].Header.Caption = 
                dayNoteCount > 0 ? "\nTagesnotizen\nOrganisation" : "\n\nOrganisation";
        }
        #endregion

        #region Planungs-Zeile

        private void BtnPlanRowPlus_Click( object sender, EventArgs e )
        {
            if ( this.planningCount < this.planningMaxCount )
            {
                // Zähler für Anzahl Planungszeilen pro MA
                this.planningCount++;

                // Loop über alle Rows in Band 0 (Abteilungen, Tagesnotizen)
                for ( int i=0; i < DataSource.Rows.Count; i++ )
                {
                    // Tagesnotizen übergehen
                    if ( ( (RowTag)DataSource.Rows[i].Tag ).RowType == RowTypes.DayNoteRow )
                        continue;

                    // Alle MA (childs) einer Abteilung
                    UltraDataRowsCollection employees = DataSource.Rows[i].GetChildRows( "Band 1" );

                    // Aktuelle Anzahl DataRows sichern
                    int currentCount = employees.Count;
                    for ( int j=0; j < currentCount; j++ )
                    {
                        // nur einmal pro MA hinzufügen
                        if ( ( (RowTag)employees[j].Tag ).TypeIndex > 0 )
                            continue;

                        // Zeilen werden dadurch alle am Schluss angefügt -> die Sortierung ordnet sie dann visuell dem Employee zu
                        this.newRowTag = new RowTag( RowTypes.PlanningRow, ( (RowTag)employees[j].Tag ).Id, planningCount - 1 );
                        UltraDataRow dataRow = employees.Add( true );
                        dataRow["Organisation"] = string.Format( "  Empl_{0}", j.ToString() );
                    }
                }
            }

            // Neue Zeilen einsortieren
            this.Grid.DisplayLayout.Bands[1].SortedColumns.RefreshSort( false );
        }

        private void BtnPlanRowMinus_Click( object sender, EventArgs e )
        {
            if ( this.planningCount > 1 )
            {
                // Eine zusätzliche Planungszeile bei allen MA (-> Band 1)
                for ( int i=0; i < DataSource.Rows.Count; i++ )
                {
                    if ( ( (RowTag)DataSource.Rows[i].Tag ).RowType == RowTypes.DayNoteRow )
                        continue;

                    // Alle Childs einer Abteilung
                    UltraDataRowsCollection employees = DataSource.Rows[i].GetChildRows( "Band 1" );
                    for ( int j=0; j < employees.Count; j++ )
                    {
                        // nur einmal pro MA hinzufügen
                        if ( ( (RowTag)employees[j].Tag ).TypeIndex == this.planningCount - 1 )
                        {
                            employees.RemoveAt( j );
                            j--;
                        }

                    }
                }
                this.planningCount--;
            }
        }
        #endregion

        #region Empl Property

        private void BtnEmplPropPlus_Click( object sender, EventArgs e )
        {
            // Zusätzliche Spalte in allen Bands
            if ( !this.DataSource.Band.Columns.Exists( "Property" ) )
            {
                this.DataSource.Band.Columns.Insert( 1, "Property", typeof( string ) );
                foreach ( UltraDataBand childBand in this.DataSource.Band.ChildBands )    // MA
                {
                    childBand.Columns.Insert( 1, "Property", typeof( string ) );
                    foreach ( UltraDataBand childOfChildBand in childBand.ChildBands )          // Dienstnotizen
                        childOfChildBand.Columns.Insert( 1, "Property", typeof( string ) );
                }
            }
            // Falls die Zeilen bereits expandiert wurden, löst das hinzufügen einer Spalte kein
            // Initialize-Row event aus -> Spalten daher immer fixieren
            this.Grid.DisplayLayout.Bands[0].Columns["Organisation"].Header.Fixed = true;
            this.Grid.DisplayLayout.Bands[0].Columns["Property"].Header.Fixed = true;
            this.Grid.DisplayLayout.Bands[1].Columns["Organisation"].Header.Fixed = true;
            this.Grid.DisplayLayout.Bands[1].Columns["Property"].Header.Fixed = true;
            this.Grid.DisplayLayout.Bands[2].Columns["Organisation"].Header.Fixed = true;
            this.Grid.DisplayLayout.Bands[2].Columns["Property"].Header.Fixed = true;
        }

        private void BtnEmplPropMinus_Click( object sender, EventArgs e )
        {
            if ( this.DataSource.Band.Columns.Exists( "Property" ) )
            {
                this.DataSource.Band.Columns.RemoveAt( 1 );
                foreach ( UltraDataBand childBand in this.DataSource.Band.ChildBands )
                {
                    childBand.Columns.RemoveAt( 1 );
                    foreach ( UltraDataBand childOfChildBand in childBand.ChildBands )
                        childOfChildBand.Columns.RemoveAt( 1 );
                }
            }
        }

        #endregion

        private void BtnToggleView_Click( object sender, EventArgs e )
        {
            this.ultraTabControl1.SelectedTab = this.ultraTabControl1.SelectedTab.Index == 0 ? 
                this.ultraTabControl1.Tabs[1] : this.ultraTabControl1.Tabs[0];

            // Variante DataSource wechseln
            /*
            if ( this.Grid.DataSource == this.DataSource )
            {
                this.viewType = ViewTypes.Detail;
                this.Grid.DataSource = this.DataSourceDetail;
            }
            else
            {
                this.viewType = ViewTypes.Standard;
                this.Grid.DataSource = this.DataSource;
            }
            */
        }

        private void BtnZB_Click( object sender, EventArgs e )
        {
            if ( this.OwnedForms.Length == 0 )
            {
                this.fZB = new FormZB();
                this.fZB.Owner = this;
                this.fZB.Show();
            }
        }

        private void FormByUltraDataSource_Deactivate( object sender, EventArgs e )
        {
            if ( this.fZB != null )
                this.fZB.Hide();
        }

        private void FormByUltraDataSource_Activated( object sender, EventArgs e )
        {
            if ( this.fZB != null )
                this.fZB.Show();
        }

        private void Grid_DoubleClickCell( object sender, DoubleClickCellEventArgs e )
        {
            if ( e.Cell.Band.Key.Equals( "Band 1" ) && !e.Cell.Column.Key.Equals( "Organisation" ) )
            {
                if ( false == e.Cell.IsInEditMode && e.Cell.CanEnterEditMode )
                {
                    this.Grid.PerformAction( UltraGridAction.EnterEditMode );
                }
            }
        }

    }
}
